@@ -1,5 +1,3 @@
-#!/usr/local/bin/python3.2
-
 from datetime import datetime
 import time
 import sys
@@ -80,9 +78,8 @@ class Wiki():
     namespaces_names = None # map from id to localised name
     namespaces_ids = None # map from canonical name to id
 
-    def __init__(self, endpoint, throttle=1):
-        ''' Create wiki client for given endpoint addres.
-        '''
+    def __init__(self, endpoint=None, throttle=1):
+        ''' Create wiki client for given endpoint addres.  '''
         self._tokens = {}
         self.endpoint = endpoint
         self.throttle = throttle
@@ -92,8 +89,7 @@ class Wiki():
         # else may be load session
 
     def load(self, filename):
-        ''' Load session information.
-        '''
+        ''' Load cookies, endpoint, namespaces. '''
         try:
             with open(filename, 'rb') as f:
                 data = pickle.load(f)
@@ -101,19 +97,21 @@ class Wiki():
                 self.endpoint = data['endpoint']
                 self.namespaces_names = data.get('namespaces_names')
                 self.namespaces_ids = data.get('namespaces_ids')
+                self.throttle = data.get('throttle', 1)
+                self.init_namespaces()
         except IOError:
             return False
         return True
 
     def save(self, filename):
-        ''' Save session information.
-        '''
+        ''' Save cookies, endpoint, namespaces '''
         with open(filename, 'wb') as f:
             data = dict(
                 cookies=self.cookies,
                 endpoint=self.endpoint,
                 namespaces_names=self.namespaces_names,
                 namespaces_ids=self.namespaces_ids,
+                throttle=self.throttle,
             )
             pickle.dump(data, f)
 
@@ -126,6 +124,7 @@ class Wiki():
 
         Returns headers and parsed json of resonce.
         '''
+        assert self.endpoint, "Can't make a request without endpoint"
         if params is None:
             params = {}
         params['format'] = 'json'
@@ -183,13 +182,14 @@ class Wiki():
     def query_cont(self, params):
         ''' Get query to the API for lists.
 
-        Returns pair of responce, and continue query params.
+        Returns pair of responce and continue query params.
         '''
         params['action'] = 'query'
         responce, content = self.request(params)
         return content['query'], content.get('query-continue')
 
     def opensearch(self, search, limit=20):
+        ''' Return a sequence of page titles which starts from given prefix '''
         responce, content = self.request(dict(
             action='opensearch',
             search=search,
@@ -198,8 +198,7 @@ class Wiki():
         return content
 
     def _get_login_cookie(self, login, password):
-        ''' Returns dictionary with cookies for given account.
-        '''
+        ''' Returns dictionary with cookies for given account.  '''
         responce, content = self.request(
             dict(
                 action='login',
@@ -237,15 +236,25 @@ class Wiki():
         else:
             raise WikiError(content['result'], 'Login error: %s.' % content['result'])
 
-    def session_login(self, login=None, password=None):
-        ''' Start session for given account.
-        '''
+    def set_endpoint(self, url):
+        self.endpoint = 'http://%s/w/api.php' % url
+
+    def session_login(self, endpoint=None, login=None, password=None):
+        ''' Start session for given account. '''
+        if self.endpoint:
+            self.set_endpoint(endpoint)
+        else:
+            if not endpoint:
+                endpoint = input('Endpoint: ')
+            self.set_endpoint(endpoint)
+
         if not login:
             login = input('Login: ')
         if not password:
             password = getpass.getpass('Password for %s: ' % login)
         try:
             self.cookies = self._get_login_cookie(login, password)
+            self.get_namespaces()
         except WikiError as e:
             if e.code == 'WrongPass':
                 print('Wrong password')
@@ -254,15 +263,13 @@ class Wiki():
                 raise
 
     def session_file(self, filename):
-        ''' Start session which is persistent in file.
-        '''
+        ''' Start session which is persistent in file. '''
         if not self.load(filename):
             self.session_login()
             self.save(filename)
         
     def get_token(self, intoken, **params):
-        ''' Get token for write operation.
-        '''
+        ''' Get token for edit operation. '''
         token = self._tokens.get(intoken)
         if not token:
             params['intoken'] = intoken
@@ -353,24 +360,64 @@ class Wiki():
             for page in res['allpages']:
                 yield Page(self, page['title'])
 
-    def recent_changes(self, start=None, end=None):
+    def logevents(self, start=None, end=None,
+        event_type=None,
+    ):
+        q = {
+            'list': 'logevents',
+            'lelimit': LIST_LIMIT,
+            'ledir': 'older', # newer changes first
+        }
+        if start and end and start < end:
+            start, end = end, start # newer changes first, remember?
+        if start:
+            q['lestart'] = datetime2zulu(start)
+        if end:
+            q['leend'] = datetime2zulu(end)
+        if event_type:
+            q['letype'] = event_type
+
+        q['leprop'] = 'user|comment|timestamp|title'
+        res, cont = self.query_cont(q)
+        for event in res['logevents']:
+            yield Change(self, event)
+
+        while cont:
+            q.update(cont['logevents'])
+            res, cont = self.query_cont(q)
+            for event in res['logevents']:
+                yield Change(self, event)
+        
+
+    def recent_changes(self, start=None, end=None,
+        change_type=None, # edit, new, log
+        namespace=None,
+    ):
         q = {
             'list': 'recentchanges',
             'rclimit': LIST_LIMIT,
+            'rcdir': 'older', # newer changes first
         }
+        if start and end and start < end:
+            start, end = end, start # newer changes first, remember?
         if start:
             q['rcstart'] = datetime2zulu(start)
         if end:
             q['rcend'] = datetime2zulu(end)
+        if namespace:
+            q['rcnamespace'] = namespace
+        if change_type:
+            q['rctype'] = change_type
+        q['rcprop'] = 'user|comment|timestamp|title|flags'
         res, cont = self.query_cont(q)
         for change in res['recentchanges']:
-            yield change
+            yield Change(self, change)
 
         while cont:
             q.update(cont['recentchanges'])
             res, cont = self.query_cont(q)
             for change in res['recentchanges']:
-                yield change
+                yield Change(self, change)
 
     def page(self, title):
         return Page(self, title)
@@ -399,10 +446,15 @@ class Wiki():
             self.namespaces_names[ns['id']] = ns['*']
             self.namespaces_ids[ns.get('canonical')] = ns['id']
 
+        self.init_namespaces()
+
+    def init_namespaces(self):
+        ''' Set some constants '''
         def ns_name(canonical):
             return self.namespaces_names[self.namespaces_ids[canonical]]
 
         self.USER_NS_NAME = ns_name('User')
+        self.USER_TALK_NS_NAME = ns_name('User talk')
         self.CATEGORY_NS_NAME = ns_name('Category')
         self.FILE_NS_NAME = ns_name('File')
 
@@ -419,6 +471,18 @@ class Wiki():
     def __str__(self):
         return 'Wiki("%s")' % self.endpoint
 
+
+class Change:
+    def __init__(self, wiki, data):
+        self.wiki = wiki
+        self.type = data.get('type')
+        self.time = zulu2datetime(data['timestamp'])
+        self.page = Page(wiki, data['title'])
+        self.user = User(wiki, data['user'])
+
+    def __str__(self):
+        return '{}.change({})'.format(self.page, self.time)
+
 class Page:
     def __init__(self, wiki, title):
         self.wiki = wiki
@@ -668,6 +732,12 @@ class User():
             self.name
         ))
 
+    def talkpage(self):
+        return Page(self.wiki, '%s:%s' % (
+            self.wiki.USER_TALK_NS_NAME,
+            self.name
+        ))
+
     def contributions(self):
         q = {
             'list': 'usercontribs',
@@ -751,16 +821,3 @@ if __name__ == '__main__':
         if i > 10:
             break
         print(change)
-
-    '''
-    filename = 'Betta splendens with bubble nest.jpg'
-    copy_file(enwiki, cybwiki, filename)
-    input('press Enter to end test')
-    cybwiki.file(filename).delete('end test')
-    '''
-    '''
-    current_text = ''
-    for revision in reversed(list(page.revisions())):
-        print_diff(current_text, revision['*'])
-        current_text = revision['*']
-    '''
